#!/bin/bash
set -eo pipefail

default_api_key_path=/var/opt/fortanix/em-agent/join-token
em_agent_conf_file=/etc/em-agent/em-agent.conf
install_enclave_os=true

if [ "$EUID" -ne 0 ]; then
    run_as_root_msg="Please run this file as root"
    echo ${run_as_root_msg}
    exit 1
fi

function usage() {
    installer_usage="USAGE:    sudo bash $0 [--no-enclave-os] <join-token> --attestation-type=<attestation-type>"
    installer_options="OPTIONS:
	--no-enclave-os				 Do not install packages needed to run Enclave OS images
	--attestation-type=<attestation-type>	 Attestation type supported are EPID or DCAP."
    echo -en ${installer_usage}
    echo -en ${installer_options}
}

function append_line_if_not_exists() {
    grep -qxF "$1" "$2" || echo "$1" >> "$2"
}

function die() {
    echo "error - see log file $log_file for full details" >&3
    echo "Tail of the log file:" >&3
    tail -n 15 "$log_file" >&3
    exit 1
}

while [ "$1" != "" ]; do
    case $1 in
        --no-enclave-os )    install_enclave_os=false
                             ;;
        -h | --help )        usage
                             exit
                             ;;
        * )                  api_key="$1"
    esac
    shift
done

if [ -z "$api_key" ]; then
    join_token_missing_msg="Error: join token argument is missing. The join token is the way by which this node will authenticate itself to Enclave Manager. Please consult the INSTALLER_README.md file for how to get a join token from Enclave Manager."
    echo -e $join_token_missing_msg
    usage
    exit 1
fi

install_packages_msg="* Installing em-agent"
packages_to_install=("em-agent")
if [ "$install_enclave_os" = true ]; then
    install_packages_msg+=" and "kmod-enclave-os-sgx""
    packages_to_install+=("kmod-enclave-os-sgx")
fi
install_packages_msg="$install_packages_msg... "

set -u
log_file=$(mktemp /var/log/node_agent_installer.XXXXXXXXX)
echo "This installation will be logged at $log_file"
exec 3>&1 4>&2
trap 'exec 2>&4 1>&3' 0 1 2 3
exec 1>"$log_file" 2>&1
set -x

# Try to read the API_KEY_PATH from /etc/em-agent/em-agent.conf if that file already exists from a previous
# installation. The em-agent.rpm package won't overwrite that config file if it already exists.
if [ -f "$em_agent_conf_file" ]; then
    source "$em_agent_conf_file"
fi
api_key_path="${API_KEY_PATH:-$default_api_key_path}"

echo -n "* Installing join token to $api_key_path... " >&3
mkdir -p "$(dirname "$api_key_path")" || die
echo "$api_key" > "$api_key_path" || die
echo "done" >&3

yum install -y wget tar yum-utils || die

function add_intel_sgx_local_repo() {
    echo "* Installing intel-sgx yum local repository... " >&3
    if [ -f "/etc/yum.repos.d/opt_sgx_rpm_local_repo.repo" ]; then
        echo -n "opt_sgx_rpm_local_repo is already enabled on your system. Do you want to overwrite it? (y|n) " >&3
        read -p "" yn_var
        case $yn_var in
            [Yy]* )
                rm -f /tmp/sgx_rpm_local_repo.tgz || true ;
                rm -rf /opt/sgx_rpm_local_repo || true ;
                rm -rf /etc/yum.repos.d/opt_sgx_rpm_local_repo.repo || true ;
                wget https://download.01.org/intel-sgx/sgx-linux/2.14/distro/rhel7.6-server/sgx_rpm_local_repo.tgz -P /tmp/ ;
                tar -xzvf /tmp/sgx_rpm_local_repo.tgz -C /opt/ ;
                yum-config-manager --add-repo file:///opt/sgx_rpm_local_repo ;;
            * )
                echo -n "Re-using the existing repo files" >&3 ;;
        esac
    else
        wget https://download.01.org/intel-sgx/sgx-linux/2.14/distro/rhel7.6-server/sgx_rpm_local_repo.tgz -P /tmp/
        tar -xzvf /tmp/sgx_rpm_local_repo.tgz -C /opt/
        yum-config-manager --add-repo file:///opt/sgx_rpm_local_repo
    fi

    echo "done" >&3
}

function add_em_agent_repo() {
    echo -n "* Installing EM Agent yum repository... " >&3
    yum-config-manager --add-repo https://download.fortanix.com/linux/yum/el7/em-agent.repo
    yum-config-manager --enable em-agent
    echo "done" >&3
}

add_intel_sgx_local_repo

add_em_agent_repo

# Install packages from intel_sgx_local_repo with nogpgkey
# Although, these packages are mentioned as dependencies for em-agent but because these packages are unsigned so
# yum install -y em-agent is complaining about these unsigned packages.
yum install --nogpgcheck -y libsgx-enclave-common libsgx-launch libsgx-urts libsgx-epid libsgx-quote-ex

echo -n "$install_packages_msg" >&3
yum install -y "${packages_to_install[@]}" || die
echo "done" >&3

echo "Installation finished" >&3
